package space.anyi.BI.config;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import java.util.concurrent.*;

/**
 * @ProjectName: serve
 * @FileName: ThreadPoolExecutorConfig
 * @Author: 杨逸
 * @Data:2024/12/7 16:01
 * @Description: 线程池配置
 */
@Configuration
public class ThreadPoolExecutorConfig {
    private final static Logger log = LoggerFactory.getLogger(ThreadPoolExecutorConfig.class);
    @Bean
    public ThreadPoolExecutor getThreadPoolExecutor(){
        //核心线程数
        int corePoolSize = 1;
        //最大线程数
        int maximumPoolSize = 2;
        //非核心线程存活空闲时间
        long keepAliveTime = 100L;
        //时间单位
        TimeUnit unit = TimeUnit.SECONDS;
        //任务队列
        BlockingQueue<Runnable> workQueue = new ArrayBlockingQueue<>(5);
        //线程工厂
        ThreadFactory threadFactory = new ThreadFactory() {
            @Override
            public Thread newThread(Runnable r) {
                Thread thread = new Thread(r);
                return thread;
            }
        };
        //拒绝策略处理器
        //RejectedExecutionHandler handler= new ThreadPoolExecutor.CallerRunsPolicy();
        RejectedExecutionHandler handler= new RejectedExecutionHandler(){
            /**
             * @param r 任务
             * @param executor 线程池
             * @description: 拒绝策略处理器
             * @author: 杨逸
             * @data:2024/12/07 16:07:54
             * @since 1.0.0
             */
            @Override
            public void rejectedExecution(Runnable r, ThreadPoolExecutor executor) {
                int activeCount = executor.getActiveCount();
                int corePoolSize = executor.getCorePoolSize();
                int maximumPoolSize = executor.getMaximumPoolSize();
                int queueSize = executor.getQueue().size();
                log.warn("任务被拒绝执行,当前工作线程数:{},核心线程数:{},最大线程数:{},队列大小:{}",activeCount,corePoolSize,maximumPoolSize,queueSize);
            }
        };
        //创建线程池
        ThreadPoolExecutor threadPoolExecutor = new ThreadPoolExecutor(corePoolSize,maximumPoolSize, keepAliveTime,unit,workQueue,threadFactory);
        return threadPoolExecutor;
    }
}
